-- Import necessary AndroLua classes for Android interaction
local luajava = require("luajava")
local Context = luajava.bindClass("android.content.Context")
local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
local Intent = luajava.bindClass("android.content.Intent")
local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
local Activity = luajava.bindClass("android.app.Activity")
local Handler = luajava.bindClass("android.os.Handler")
local Toast = luajava.bindClass("android.widget.Toast")
local TextToSpeech = luajava.bindClass("android.speech.tts.TextToSpeech")
local Locale = luajava.bindClass("java.util.Locale")
local Vibrator = luajava.bindClass("android.os.Vibrator")
local URL = luajava.bindClass("java.net.URL")
local InputStreamReader = luajava.bindClass("java.io.InputStreamReader")
local BufferedReader = luajava.bindClass("java.io.BufferedReader")

local activity = this

-- Initialize Text-to-Speech
local tts
tts = TextToSpeech(activity, luajava.createProxy("android.speech.tts.TextToSpeech$OnInitListener", {
    onInit = function(status)
        if status == TextToSpeech.SUCCESS then
            tts.setLanguage(Locale.getDefault()) -- Set TTS language to device default
        end
    end
}))

-- Function to vibrate device
function vibrate(pattern)
    local vibrator = activity.getSystemService(Context.VIBRATOR_SERVICE)
    vibrator.vibrate(luajava.createArray("long", pattern), -1) -- -1 means no repeat
end

-- Function to fetch and print articles from detik.com
function fetchArticles(query)
    local searchUrl = "https://www.detik.com/search/searchall?query=" .. query
    local url = URL(searchUrl)
    local connection = url.openConnection()
    local inputStream = InputStreamReader(connection.getInputStream())
    local bufferedReader = BufferedReader(inputStream)
    
    local line
    local content = {}
    while true do
        line = bufferedReader.readLine()
        if line == nil then break end
        table.insert(content, line)
    end
    bufferedReader.close()
    
    -- Parse and print the article titles
    local articlesFound = false
    for _, line in ipairs(content) do
        local title = line:match('<title>(.-)</title>')
        if title then
            print(title)
            articlesFound = true
        end
    end
    
    if not articlesFound then
        print("Tidak ada artikel ditemukan untuk pencarian: " .. query)
    end
end

-- Function to start voice recognition with the default device language
function startVoiceRecognition(activity)
    local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    
    -- Use the default device language
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault().toString())

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
        onResults = function(params)
            local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            -- Handle the recognized speech results
            local recognizedText = results.get(0):lower()

            -- Speak the message using TTS
            local ttsMessage = "Kamu mencari artikel tentang: " .. recognizedText .. ". Mohon tunggu sebentar yaa..."
            tts.speak(ttsMessage, TextToSpeech.QUEUE_FLUSH, nil, "MessageID")

            -- Vibrate to indicate success
            vibrate({0, 2000})

            -- Fetch and print articles
            fetchArticles(recognizedText)

            return true
        end,
        onError = function(error)
            -- Handle the error event
            local errorMessage = "Aku tidak dapat mengenali ucapan dirimu, coba ucapkan dengan jelas."
            if error == SpeechRecognizer.ERROR_NETWORK or error == SpeechRecognizer.ERROR_NETWORK_TIMEOUT then
                errorMessage = errorMessage .. " Tidak ada koneksi internet."
                tts.speak(errorMessage, TextToSpeech.QUEUE_FLUSH, nil, "ErrorID")
                vibrate({0, 700})
            else
                tts.speak(errorMessage, TextToSpeech.QUEUE_FLUSH, nil, "ErrorID")
                vibrate({0, 100, 100, 100, 100, 100}) -- Short vibrations 5 times
            end
        end,
        onReadyForSpeech = function(params) end,
        onBeginningOfSpeech = function() 
            -- Vibrate to indicate listening started
            vibrate({0, 500})
        end,
        onBufferReceived = function(buffer) end,
        onEndOfSpeech = function() end,
        onEvent = function(eventType, params) end,
        onPartialResults = function(partialResults) end,
        onRmsChanged = function(rmsdB) end
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
end

startVoiceRecognition(activity)

-- Called when the activity is created
function onCreate(savedInstanceState)
    activity.setTitle("Voice Input Example")
end